/* 
 * %Z%%W% %I%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * IBM Java(tm)2 SDK, Standard Edition, v 1.4 
 *
 * (C) Copyright IBM Corp. 1998 , 2002. All Rights Reserved
 * (C) Copyright Sun Microsystems Inc. 1997 , 2002. All Rights Reserved
 * ===========================================================================
 */


package com.sun.tools.example.debug.bdi;

import com.sun.jdi.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

//### Should handle target VM death or connection failure cleanly.

public class ThreadInfo {

    private ThreadReference thread;
    private int status;

    private List frames;
    private int frameCount;
    private int extent;
    private int initial;
    private int prefetch;

    //private int frameIndex;

    Object userObject;  // User-supplied annotation.

    private boolean interrupted = false;

    private void assureInterrupted() throws VMNotInterruptedException {
	if (!interrupted) {
	    throw new VMNotInterruptedException();
	}
    }
    
    ThreadInfo(ThreadReference thread, int initial, int prefetch) {
	this.thread = thread;
	this.initial = initial;
	this.prefetch = prefetch;
	this.frameCount = 0;
	this.frames = null;
	//this.frameIndex = 0;
    }

    public ThreadReference thread() {
	return thread;
    } 

    public int getStatus() throws VMNotInterruptedException {
        assureInterrupted();
	update();
	return status;
    }

    public int getFrameCount() throws VMNotInterruptedException {
        assureInterrupted();
	update();
	return frameCount;
    }

    public StackFrame getFrame(int index) throws VMNotInterruptedException {
        assureInterrupted();
	update();
	try {
	    if (extent < index) {
		if (index < frameCount) {
		    int newExtent = Math.min(index + prefetch, frameCount - 1);
		    frames.addAll(thread.frames(extent + 1, newExtent - extent));
		    extent = newExtent;
		} else {
		    throw new FrameIndexOutOfBoundsException();
		}
	    }
	    return (StackFrame)frames.get(index);
	} catch (IncompatibleThreadStateException e) {
	    // Should not happen
	    interrupted = false;
	    throw new VMNotInterruptedException();
	}
    }

    public Object getUserObject() {
	return userObject;
    }

    public void setUserObject(Object obj) {
	userObject = obj;
    }

    //### This doesn't belong in BDI -- current frame is a UI policy issue.
    /*******************
    public int getCurrentFrameIndex() {
	return frameIndex;
    }

    public void setCurrentFrameIndex(int index) {
	if (index < frameCount) {
	    frameIndex = index;
	} else {
	    throw new FrameIndexOutOfBoundsException();
	}
    }

    public StackFrame getCurrentFrame() throws VMNotInterruptedException {
	return getFrame(frameIndex);
    }
    *******************/

    // Refresh upon first access after cache is cleared.

    void update() throws VMNotInterruptedException {
	if (frames == null) {
	    try {
		status = thread.status();
		frameCount = thread.frameCount();
		frames = new ArrayList(thread.frames(0, Math.min(frameCount, initial)));
		extent = initial - 1;
	    } catch (IncompatibleThreadStateException e) {
		// Should not happen
		interrupted = false;
		throw new VMNotInterruptedException();
	    }
	}
    }
    
    // Called from 'ExecutionManager'.
    
    void validate() {
	interrupted = true;
    }

    void invalidate() {
	interrupted = false;
	frames = null;
	//frameIndex = 0;
	status = ThreadReference.THREAD_STATUS_UNKNOWN;
    }
    
}
